/***************************************************************************

    @(#)$Id: wmr918util.c,v 1.18 2004/01/17 21:45:44 twitham Exp $

                          wmr918util.c  -  description
                             -------------------
    begin                : Mon May 28 2001
    copyright            : (C) 2001 by Dominique Le Foll
    email                : dominique@le-foll.demon.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "wx200.h"
#include "wmr918.h"
#include "parse.h"

/*
 * Space to hold baro trend monitor for 60 minutes
 */
#define BARO_TREND_MINUTES 60
int baro_trend[BARO_TREND_MINUTES];

unsigned char wmr918groupbuf[WMR918GROUPS][WMR918GROUP_LENMAX];
int wmr918grouplen[] = WMR918GROUP_LENGTHS;
MINMAXDATA minmax;

void
checksum (unsigned char *buffer,int grouptype)
{ // calculate the checksum for the wx200 buffer
  int chksum, i;
  chksum = 0;
  for (i=0;i<=(grouplen[grouptype])-2;i++) // for all byte except the checksum itself
    chksum += (int) buffer[i];
  buffer[grouplen[grouptype]-1] = (unsigned char) (chksum & 0x00ff);
}

void
timetobuffer(unsigned char *p, time_t nowtime, int halfbyte)
{ // print unixtime in targetbuffer
  unsigned char clock[7]; // used for hh, mm, ss conversion
  struct tm *now = localtime(&nowtime);
  sprintf (clock,"%2u%2u%2u", now->tm_min, now->tm_hour, now->tm_mday);
  clock[ 6 ] = now->tm_mon + 1;
  if( ! halfbyte ){
    p[0] = MHI( clock[0] ) | LO( clock[1] );
    p[1] = MHI( clock[2] ) | LO( clock[3] );
    p[2] = MHI( clock[4] ) | LO( clock[5] );
    p[3] = MHI( HI(p[3]) ) | LO( clock[6] );
  } else {
    p[0] = MHI( clock[1] ) | LO(     p[0] );
    p[1] = MHI( clock[3] ) | LO( clock[0] );
    p[2] = MHI( clock[5] ) | LO( clock[2] );
    p[3] = MHI( clock[6] ) | LO( clock[4] );
  }
}

void
wmr918initminmax()
{
  time_t nowtime = time(NULL);
  minmax.inhumid.lowtime = nowtime;
  minmax.inhumid.hightime = nowtime;
  minmax.inhumid.lowsign = 0;
  minmax.inhumid.highsign = 1;
  minmax.inhumid.lowvalue = 0x7F000000;
  minmax.inhumid.highvalue = 0x7F000000;
  minmax.outhumid = minmax.inhumid;
  minmax.intemp = minmax.inhumid;
  minmax.outtemp = minmax.inhumid;
  minmax.indew = minmax.inhumid;
  minmax.outdew = minmax.inhumid;
  minmax.windgust = minmax.inhumid;
  minmax.windchill = minmax.inhumid;
}

void
updateminmax( MINMAX *mm, time_t now, int value, int sign )
{
  if( ( value * ((sign)?(-1):(1)) ) > 
      ( mm->highvalue * ((mm->highsign)?(-1):(1)) ) ){
    mm->highvalue = value;
    mm->highsign = sign;
    mm->hightime = now;
  }
  if( ( value * ((sign)?(-1):(1)) ) < 
      ( mm->lowvalue * ((mm->lowsign)?(-1):(1)) ) ){
    mm->lowvalue = value;
    mm->lowsign = sign;
    mm->lowtime = now;
  }
}

int
wmr918parse (int grouptype)   // convert the wmr918 buffer in wx200 buffer
{
  unsigned char *buf200a, *buf200b, *buf200c;  // short to wx200 buffer
  unsigned char *buf918; // short to wmr918 buffer

  static int ready = 0;		/* have a complete set of data yet? */
  static int prev_baro = -1;	/* previous baro reading, for tendency */
  int i, baro, baros, trend, wx200type = -1, group, ret, tmp;
  unsigned char pressure[4]; //coded abcd  -> abcd
  unsigned char seapressure[7];//coded abc.d or abcd.ef -> abcd.e
  unsigned char clock[2]; // used for hh, mm, ss conversion
  float barosea;
  struct tm *now = NULL;
  time_t nowtime = time(NULL);

/*  	    if (verbose) */
/*  	    	printf ("wx200d debug WMR918 GROUPNUM = %d \n",grouptype); */
  switch (grouptype) {
  case WMR918GROUP0:         // anemometer and wind related data
    {
      buf918 = wmr918groupbuf[WMR918GROUP0];
      buf200a = groupbuf[GROUPC]; // equivalent to GROUPC

      // pass speed and direction to update function
      updateminmax( &minmax.windgust, nowtime, (buf918[4] << 20)|(HI(buf918[3])<<16)|(LO(buf918[3])<<8)|(buf918[2]), 0 );
      // insert 0xF between speed and direction so a higher dir doesn't update the highvalue
      minmax.windgust.highvalue = minmax.windgust.highvalue | 0xF000;
      // update min windchill
      updateminmax( &minmax.windchill, nowtime, buf918[7], BIT(HI(buf918[6]),3) );

      // write gust speed, gust dir and average speed
      buf200a[1] = MHI(LO(buf918[4])) | HI(buf918[3]);
      buf200a[2] = MHI(LO(buf918[2])) | HI(buf918[4]);
      buf200a[3] = MHI(LO(buf918[3])) | HI(buf918[2]);
      buf200a[4] = buf918[5];
      // average direction not provided uses gust in place
      buf200a[5] = MHI(LO(buf918[2])) | (LO(buf918[6]));
      buf200a[6] = MHI(LO(buf918[3])) | (HI(buf918[2]));
      
      buf200a[14] = buf200a[13] = 0;   // alarm not supported by WMR 918

      // write max and dir of windgust
      buf200a[7] = ( (minmax.windgust.highvalue >> 16) & 0xFF );
      buf200a[8] = LO(minmax.windgust.highvalue >> 24) | MHI(minmax.windgust.highvalue);
      buf200a[9] = ( (minmax.windgust.highvalue >> 4) & 0xFF );
      timetobuffer( ( buf200a + 10 ), minmax.windgust.hightime, 0 );

      buf200a[15] = 0x80; // speeds are always in m/s
      // write windchill
      buf200a[16] = buf918[7];

      // write sign of chill and low chill
      buf200a[21] = MHI((BIT(HI(buf918[6]), 3) << 1)|minmax.windchill.lowsign);

      // write low of windchill
      buf200a[17] = minmax.windchill.lowvalue;
      timetobuffer( buf200a + 18, minmax.windchill.lowtime, 0 );

      // functions not provided by WMR918
      for (i=22;i<=25;i++)
        buf200a[i] = 0;
      buf200a[23] = MHI(LO(wmr918groupbuf[WMR918GROUPF][1]));// batterie status
      checksum (buf200a,GROUPC);
      wx200type = GROUPC;
      ready |= 0x01;
      break;
    }
  case WMR918GROUP1:         // rain guage
    { buf918 = wmr918groupbuf[WMR918GROUP1];
      buf200a = groupbuf[GROUPB]; // equivalent to GROUPB

      buf200a[1] = buf918[2];
      buf200a[2] = buf918[3];
      buf200a[3] = buf918[6];
      buf200a[4] = buf918[7];
      buf200a[5] = buf918[4];
      buf200a[6] = buf918[5];
      buf200a[7] = buf918[8];
      buf200a[8] = buf918[9];
      buf200a[9] = buf918[10];
      buf200a[10] = NUM(buf918[11]) & 0xff; //wmr918 is always in mm
      // nor supported by WMR918
      buf200a[11] = 0;
      buf200a[12] = 0;

      checksum (buf200a,GROUPB);
      wx200type = GROUPB;
      ready |= 0x02;
      break;
    }
  case WMR918GROUP2:         // extra sensors
    { wx200type = -1;       // no known equivalent
      break;
    }
  case WMR918GROUP3:         // outside temp, humidity and dewpoint
    {
      buf918 = wmr918groupbuf[WMR918GROUP3];
      buf200a = groupbuf[GROUP8];// part equivalent GROUP9, GROUP8 and GROUPF
      buf200b = groupbuf[GROUP9];
      buf200c = groupbuf[GROUPA];

      //update minmax values
      updateminmax( &minmax.outtemp, nowtime, (LO(buf918[3])<<8)|(buf918[2]), BIT(HI(buf918[3]),3) );
      updateminmax( &minmax.outhumid, nowtime, buf918[4], 0 );
      updateminmax( &minmax.outdew, nowtime, buf918[5], BIT(HI(buf918[1]), 3));

      //write outside temperature
      buf200b[16] = buf918[2];
      buf200b[17] = MHI(minmax.outtemp.highvalue)|(HI(buf918[3])&0x08)|LO(buf918[3]);
      buf200b[18] = ( minmax.outtemp.highvalue >> 4 )|( minmax.outtemp.highsign << 3 );
      timetobuffer( ( buf200b + 19 ), minmax.outtemp.hightime, 0 );
      buf200b[22] = LO(buf200b[22]) | MHI( minmax.outtemp.lowvalue );
      buf200b[23] = ( minmax.outtemp.lowvalue >> 4 )|( minmax.outtemp.lowsign << 7 );
      timetobuffer( ( buf200b + 24 ), minmax.outtemp.lowtime, 0 );

      //write outside humidity
      buf200a[20] = buf918[4];
      buf200a[21] = minmax.outhumid.highvalue;
      timetobuffer( ( buf200a + 22 ), minmax.outhumid.hightime, 0 );
      buf200a[25] = LO( buf200a[25] ) | MHI( minmax.outhumid.lowvalue );
      buf200a[26] = HI( minmax.outhumid.lowvalue );
      timetobuffer( ( buf200a + 26 ), minmax.outhumid.lowtime, 1 );

      //write outside dewpoint
      buf200c[18] = buf918[5];
      buf200c[28] = (buf918[1] & 0x80) | (minmax.outdew.lowsign << 6);
      buf200c[19] = minmax.outdew.highvalue;
      timetobuffer( ( buf200c + 20 ), minmax.outdew.hightime, 0 );
      buf200c[23] = LO( buf200c[23] ) | MHI( minmax.outdew.lowvalue );
      buf200c[24] = HI( minmax.outdew.lowvalue );
      timetobuffer( ( buf200c + 24 ), minmax.outdew.lowtime, 1 );

      checksum(buf200a,GROUP8);
      checksum(buf200b,GROUP9);
      checksum(buf200c,GROUPA);
      wx200type = GROUPS;
      ready |= 0x04;
      break;
    }
  case WMR918GROUP5:         // inside temp, humidity, dewpoint, and baro
    {
      buf918 = wmr918groupbuf[WMR918GROUP5];
      buf200a = groupbuf[GROUP9];    // part equivalent to GROUP9, GROUP8 and GROUPA
      buf200b = groupbuf[GROUP8];
      buf200c = groupbuf[GROUPA];

      //update minmax values
      updateminmax( &minmax.intemp, nowtime, (LO(buf918[3])<<8)|(buf918[2]), BIT(HI(buf918[3]),3) );
      updateminmax( &minmax.inhumid, nowtime, buf918[4], 0 );
      updateminmax( &minmax.indew, nowtime, buf918[5], 0 );

      //write inside temperature
      buf200a[1] = buf918[2];
      buf200a[2] = MHI(minmax.intemp.highvalue)|(HI(buf918[3])&0x08)|LO(buf918[3]);
      buf200a[3] = ( minmax.intemp.highvalue >> 4 )|( minmax.intemp.highsign << 3 );
      timetobuffer( ( buf200a + 4 ), minmax.intemp.hightime, 0 );
      buf200a[7] = LO(buf200a[7]) | MHI( minmax.intemp.lowvalue );
      buf200a[8] = ( minmax.intemp.lowvalue >> 4 )|( minmax.intemp.lowsign << 7 );
      timetobuffer( ( buf200a + 9 ), minmax.intemp.lowtime, 0 );

      //write inside humidity
      buf200b[8] = buf918[4];
      buf200b[9] = minmax.inhumid.highvalue;
      timetobuffer( ( buf200b + 10 ), minmax.inhumid.hightime, 0 );
      buf200b[13] = LO( buf200b[13] ) | MHI( minmax.inhumid.lowvalue );
      buf200b[14] = HI( minmax.inhumid.lowvalue );
      timetobuffer( ( buf200b + 14 ), minmax.inhumid.lowtime, 1 );

      //write inside dewpoint
      buf200c[7] = buf918[5];
      buf200c[8] = minmax.indew.highvalue;
      timetobuffer( ( buf200c + 9 ), minmax.indew.hightime, 0 );
      buf200c[12] = LO( buf200c[12] ) | MHI( minmax.indew.lowvalue );
      buf200c[13] = HI( minmax.indew.lowvalue );
      timetobuffer( ( buf200c + 13 ), minmax.indew.lowtime, 1 );

      baro = buf918[6]+795;   // pressure in mb
      sprintf(pressure,"%4u",baro);
      buf200c[1] = (MHI(pressure[2] & 0x0F)) | (pressure[3] & 0x0F);
      buf200c[2] = (MHI(pressure[0] & 0x0F)) | (pressure[1] & 0x0F);

      // compute a diff between this baro and prev, use as part of a 'trend'
      // store one trend data point per minute, for the length of the trend
      i = ( (nowtime / 60) % BARO_TREND_MINUTES );

      // trend code: 0 = steady, 1 = rising, -1 = falling.  Then, we can add
      // up all the trend values in the period and if > 0 we have an overall
      // rising trend, if < 0 a falling trend, and if == 0 a steady trend
      if (prev_baro < 0 || ((baro - prev_baro) == 0)) {
          baro_trend[ i ] = 0;
      } else if ((baro - prev_baro) > 0) {
          baro_trend[ i ] = 1;
      } else {
          baro_trend[ i ] = -1;
      }
      prev_baro = baro;

      switch (LO(buf918[7]))
      { 
          case 0x0c:
             buf200c[6] = 1; //clear
          break;
          case 0x06:
             buf200c[6] = 4; //partly cloudy
          break;
          case 0x02:
             buf200c[6] = 2; //cloudy
          break;
          case 0x03:
             buf200c[6] = 8; //rain
          break;
      }       

      // See how the baro trend is faring
      trend = 0;
      for (i=0; i<BARO_TREND_MINUTES; i++) {
          trend += baro_trend[i];
      }

      // Include it into the Group A info
      // Trend: 1=Rising, 2=Steady, 4=Falling
      if (trend == 0) {
          buf200c[6] |= 0x20;
      } else if (trend > 0) {
          buf200c[6] |= 0x10;
      } else {
          buf200c[6] |= 0x40;
      }

      // see level references format abc.d to be added to baro
      seapressure[0] = HI(buf918[9]) | 0x30; // a
      seapressure[1] = LO(buf918[9]) | 0x30; // b
      seapressure[2] = HI(buf918[8]) | 0x30; // c
      seapressure[3] = LO(buf918[8]) | 0x30; // d
      sscanf (seapressure,"%4u",&baros);
      barosea = baro + baros/10.0 - 795;              //sea ref is formated as abc.d
      sprintf(seapressure,"%6.1f",barosea);
      buf200c[3] = (MHI(seapressure[3] & 0x0F)) | (seapressure[5] & 0x0F);
      buf200c[4] = (MHI(seapressure[1] & 0x0F)) | (seapressure[2] & 0x0F);
      buf200c[5] = 0x20 | seapressure[0];     //wmr918 is always in mb

      checksum(buf200a,GROUP9);
      checksum(buf200b,GROUP8);
      checksum(buf200c,GROUPA);
      wx200type = GROUPS;
      ready |= 0x08;
      break;
    }
  case WMR918GROUP6:         // inside temp, humidity, dewpoint, and baro for WMR968
    {
      buf918 = wmr918groupbuf[WMR918GROUP6];
      buf200a = groupbuf[GROUP9];    // part equivalent to GROUP9, GROUP8 and GROUPA
      buf200b = groupbuf[GROUP8];
      buf200c = groupbuf[GROUPA];

      //update minmax values
      updateminmax( &minmax.intemp, nowtime, (LO(buf918[3])<<8)|(buf918[2]), BIT(HI(buf918[3]),3) );
      updateminmax( &minmax.inhumid, nowtime, buf918[4], 0 );
      updateminmax( &minmax.indew, nowtime, buf918[5], 0 );

      //write inside temperature
      buf200a[1] = buf918[2];
      buf200a[2] = MHI(minmax.intemp.highvalue)|(HI(buf918[3])&0x08)|LO(buf918[3]);
      buf200a[3] = ( minmax.intemp.highvalue >> 4 )|( minmax.intemp.highsign << 3 );
      timetobuffer( ( buf200a + 4 ), minmax.intemp.hightime, 0 );
      buf200a[7] = LO(buf200a[7]) | MHI( minmax.intemp.lowvalue );
      buf200a[8] = ( minmax.intemp.lowvalue >> 4 )|( minmax.intemp.lowsign << 7 );
      timetobuffer( ( buf200a + 9 ), minmax.intemp.lowtime, 0 );

      //write inside humidity
      buf200b[8] = buf918[4];
      buf200b[9] = minmax.inhumid.highvalue;
      timetobuffer( ( buf200b + 10 ), minmax.inhumid.hightime, 0 );
      buf200b[13] = LO( buf200b[13] ) | MHI( minmax.inhumid.lowvalue );
      buf200b[14] = HI( minmax.inhumid.lowvalue );
      timetobuffer( ( buf200b + 14 ), minmax.inhumid.lowtime, 1 );

      //write inside dewpoint
      buf200c[7] = buf918[5];
      buf200c[8] = minmax.indew.highvalue;
      timetobuffer( ( buf200c + 9 ), minmax.indew.hightime, 0 );
      buf200c[12] = LO( buf200c[12] ) | MHI( minmax.indew.lowvalue );
      buf200c[13] = HI( minmax.indew.lowvalue );
      timetobuffer( ( buf200c + 13 ), minmax.indew.lowtime, 1 );

      baro = (buf918[6] | (BIT(buf918[7],0) << 8)) + 600;   // pressure in mb
      sprintf(pressure,"%4u",baro);
      buf200c[1] = (MHI(pressure[2] & 0x0F)) | (pressure[3] & 0x0F);
      buf200c[2] = (MHI(pressure[0] & 0x0F)) | (pressure[1] & 0x0F);

      // compute a diff between this baro and prev, use as part of a 'trend'
      // store one trend data point per minute, for the length of the trend
      i = ( (nowtime / 60) % BARO_TREND_MINUTES );

      // trend code: 0 = steady, 1 = rising, -1 = falling.  Then, we can add
      // up all the trend values in the period and if > 0 we have an overall
      // rising trend, if < 0 a falling trend, and if == 0 a steady trend
      if (prev_baro < 0 || ((baro - prev_baro) == 0)) {
          baro_trend[ i ] = 0;
      } else if ((baro - prev_baro) > 0) {
          baro_trend[ i ] = 1;
      } else {
          baro_trend[ i ] = -1;
      }
      prev_baro = baro;
      
      switch (HI(buf918[7]))
      {
            case 0x0c:
                  buf200c[6] = 1; //clear
            break;
            case 0x06:
                  buf200c[6] = 4; //partly cloudy
            break;
            case 0x02:
                  buf200c[6] = 2; //cloudy
            break;
            case 0x03:
                  buf200c[6] = 8; //rain
            break;
      }

      // See how the baro trend is faring
      trend = 0;
      for (i=0; i<BARO_TREND_MINUTES; i++) {
          trend += baro_trend[i];
      }

      // Include it into the Group 9 info
      // Trend: 1=Rising, 2=Steady, 4=Falling
      if (trend == 0) {
          buf200c[6] |= 0x20;
      } else if (trend > 0) {
          buf200c[6] |= 0x10;
      } else {
          buf200c[6] |= 0x40;
      }

      // see level references format abcd.ef to be added to baro
      // only abcd.e can be used for compatibility with wx200

      seapressure[0] = HI(buf918[10]) | 0x30; // a
      seapressure[1] = LO(buf918[10]) | 0x30; // b
      seapressure[2] = HI(buf918[9]) | 0x30; // c
      seapressure[3] = LO(buf918[9]) | 0x30; // d
      seapressure[4] = HI(buf918[8]) | 0x30; // e
      seapressure[5] = LO(buf918[8]) | 0x30; // f
      sscanf (seapressure,"%6u",&baros);
      barosea = baro + baros/100.0 - 600;             //sea ref is formated as abc.d
      sprintf(seapressure,"%6.1f",barosea); //leave 1/10 for compatibility
      buf200c[3] = (MHI(seapressure[3] & 0x0F)) | (seapressure[5] & 0x0F);
      buf200c[4] = (MHI(seapressure[1] & 0x0F)) | (seapressure[2] & 0x0F);
      buf200c[5] = 0x20 | seapressure[0];     //wmr918 is always in mb

      checksum(buf200a,GROUP9);
      checksum(buf200b,GROUP8);
      checksum(buf200c,GROUPA);
      wx200type = GROUPS;
      ready |= 0x08;
      break;
    }
  case WMR918GROUPE:			// sequence number ( minutes )
  case WMR918GROUPF:      // hourly report ( hour )
    {                     // part equivalent to GROUPF and GROUP8
      // wmr918 clock is not used but system clock instead
     	buf200a = groupbuf[GROUP8];
     	buf200b = groupbuf[GROUPF];
     	   	
     	now = localtime(&nowtime);
     	wx.td.clock.sec = now->tm_sec;
     	sprintf (clock,"%2u",now->tm_sec);
     	buf200a[1] = (MHI(clock[0] & 0x0F)) | (clock[1] & 0x0F);
     	wx.td.clock.min = now->tm_min;
     	sprintf (clock,"%2u",now->tm_min);
     	buf200a[2]= (MHI(clock[0] & 0x0F)) | (clock[1] & 0x0F);
     	wx.td.clock.hour = now->tm_hour;
     	sprintf (clock,"%2u",now->tm_hour);
     	buf200a[3] = (MHI(clock[0] & 0x0F)) | (clock[1] & 0x0F);
     	wx.td.clock.day = now->tm_mday;
     	sprintf (clock,"%2u",now->tm_mday);
     	buf200a[4] = (MHI(clock[0] & 0x0F)) | (clock[1] & 0x0F);
     	wx.td.clock.mon = now->tm_mon + 1; //month range 0-11 under unix
     	buf200a[5] = (MHI(0x01)) | (wx.td.clock.mon); // 24 hours / DD MM format hardwired

     	// copy date and time to F type for wx200 client compatibility
     	memcpy(&buf200b[1], &buf200a[1], 3);

	checksum(buf200b,GROUPF);
	checksum(buf200a,GROUP8);
	wx200type = GROUP8;
	ready |= 0x10;
	break;
    }

  default:
    {     // ignore any other groupe type
		wx200type = -1;
    break;
		}
  }
  if ((ready & 0x1f) != 0x1f)
    return -3;

  ret = -3;			/* return positive only if any bit changed */
  group = wx200type;
  if (group < 0)
    return -3;
  if (group == GROUP8) {	/* only a time change? */
    if (!memcmp(&groupbuf[GROUP8][4], &groupbuf[GROUP8 + GROUPS][4], 30))
      group = GROUPF;
  }
  if (group == GROUPS) {	/* HACK: multiple group change */
    ret = 0;
    for (i = 0; i < 3; i++) {
      group = GROUP8 + i;
      if (memcmp(groupbuf[group], groupbuf[group + GROUPS], grouplen[group]))
	ret |= (0x01 << i);	/* set bits of changed groups */
      memcpy(groupbuf[group + GROUPS], groupbuf[group], grouplen[group]);
    }
  } else {
    if (memcmp(groupbuf[group], groupbuf[group + GROUPS], grouplen[group]))
      ret = group;
    memcpy(groupbuf[group + GROUPS], groupbuf[group], grouplen[group]);
  }
  return ret;
}

int
wmr918bufread (int fd)
{
  unsigned char syncbuf[3];
  int retval, i, grouptype, cs;

  /* read the first three bytes -- ff ff type */
  retval = read(fd, &syncbuf[0], 3 );
  if (!retval) return EOF;	/* no more data */
  if (retval !=3)
    return (-2);              // error reading serial port
  while ((syncbuf[0] != 0xff) || (syncbuf[1] != 0xff)
         || (syncbuf[2] > 15)) {
    syncbuf[0] = syncbuf[1];
    syncbuf[1] = syncbuf[2];
    retval = read(fd, &syncbuf[2], 1 );
    if (retval !=1)
      return (-2);              // error reading serial port
  }
  grouptype =  (int)syncbuf[2];
  wmr918groupbuf[grouptype][0] = grouptype; // place group type

  // read remaining bytes of this type + checksum
  retval = read(fd, &wmr918groupbuf[grouptype][1],(wmr918grouplen[grouptype]));
  if (retval != wmr918grouplen[grouptype])
    return (-2);

  // Control checksum
  cs = 2*0xff;                // sync FF are part of the Chksum
  for (i=0; i<wmr918grouplen[grouptype]; i++)
    cs += wmr918groupbuf[grouptype][i];
  cs = cs & 0xFF;
  if (cs != wmr918groupbuf[grouptype][(wmr918grouplen[grouptype])])
    return (-2);       // checksum is wrong

  // checksum is OK parse data and return wx200 group type
  return (wmr918parse(grouptype));
}
