/*
 * @(#)$Id: wxfilter.c,v 1.10 2004/01/24 23:04:25 twitham Exp $
 *
 * Copyright (C) 1998 - 2004 Tim Witham <twitham@surewest.net>
 *
 * (see the files README and COPYING for more details)
 *
 * This file implements a conversion filter (stdin to stdout).
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <getopt.h>
#include "wx200.h"

int wx200stationtype = 0;

int
stationbufread(int fd)
{
  if (wx200stationtype == 0)	/* original WX200/WM918 */
    return wx200bufread(fd);
  else if (wx200stationtype == 1) /* WMR918 by Dominique Le Foll */
    return wmr918bufread(fd);
  return -1;			/* huh? */
}

int
main(int argc, char **argv) {
  const char *flags = "Hhrz";
  unsigned int opt, bufnum, opt_z = 0, i, j, bit, bits;

  while ((opt = getopt(argc, argv, flags)) != EOF) {
    switch (opt) {
    case '?':
    case 'h':
    case 'H':
      printf("usage: wxz [-h | -z]\n\
	-h	show this help and exit\n\
	-z	produce abbreviated binary, instead of tab-delimited ASCII\n");
      exit(0);
    case 'r':
      wx200stationtype = 1;
      break;
    case 'z':
      opt_z = 1;
    }
  }

  wx200bufinit();
  wx200tabinit();
  while ((bufnum = stationbufread(0)) != EOF) {
    switch (bufnum) {
    case GROUP1:
    case GROUP2:
    case GROUP3:
    case GROUP4:		/* 1-7 is a WMR918 multi-group (3 bits) */
    case GROUP5:
    case GROUP6:
    case GROUP7:
    case GROUP8:
    case GROUP9:
    case GROUPA:
    case GROUPB:
    case GROUPC:
    case GROUPF:
	if (bufnum < GROUP8) {	/* WMR918 tweaks up to 3 groups at once */
	  bits = bufnum;
	  bufnum = GROUP8;
	  j = GROUPA;
	} else {		/* otherwise, just log this one group */
	  bits = 0x01;
	  j = bufnum;
	}
	for (i = bit = 0; bufnum <= j; bufnum++) {
	  if (bits & (0x01 << bit)) { /* this group changed? */
	    if (opt_z)		/* compress */
	      write(1, groupbuf[bufnum], grouplen[bufnum]);
	    else if (wx200tab(bufnum)) /* tab-delimit */
	      write(1, tabbuf, strlen(tabbuf));
	  }
	  bit++;
	}
    }
  }
  exit(0);
}
